package com.nisum.Employeeinfo.controller;

import com.nisum.Employeeinfo.dto.EmployeeDto;
import com.nisum.Employeeinfo.mapper.EmployeeMapper;
import com.nisum.Employeeinfo.model.Employee;
import com.nisum.Employeeinfo.service.EmployeeService;
import com.nisum.Employeeinfo.service.EmployeeServiceImpl;
import com.nisum.Employeeinfo.validator.CommonServiceValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/employee")
public class EmployeeController {

    @Autowired
    private EmployeeService employeeService;

    @Autowired
    private CommonServiceValidator validator;

    private static  final Logger log= LoggerFactory.getLogger(EmployeeController.class);

    @PostMapping("/addEmployee")
    public Mono<ResponseEntity<Employee>> addEmployee(@RequestBody Mono<EmployeeDto> mono){
//        return employeeService.addEmployee(mono)
//                .map(ResponseEntity::ok);
        return mono
                .doOnNext(employeeDto -> log.info("validating incoming employee data: {}",employeeDto))
                .flatMap(employeeDto -> {
                    Employee employee= EmployeeMapper.toEntity(employeeDto);
                    return validateEmployee(employee)
                            .then(employeeService.addEmployee(Mono.just(employeeDto)));
                })
                .map(ResponseEntity::ok)
                .doOnError(error -> log.error("Error while adding employee: {}", error.getMessage()));

    }
    @GetMapping("/getEmployeeById/{id}")
    public Mono<ResponseEntity<Employee>> getEmployeeById(@PathVariable Integer id){

//        return employeeService.getEmployeeById(id)
//                .doOnNext(employee -> log.info("received employee {}",employee));
        return validator.isEmployeeIdValid(id)
                .flatMap(valid -> employeeService.getEmployeeById(id))
                .map(ResponseEntity::ok)
                .doOnError(error -> log.error("Error fetching employee by ID: {}", error.getMessage()));
    }

    @GetMapping("/getAllEmployee")
    public Flux<Employee> getAllEmployee(){
        return employeeService.getAllEmployee()
                .doOnNext(employee -> log.info("Processing employee data: {}", employee))
                .doOnError(error -> log.error("Error fetching employees: {}", error.getMessage()));

    }

    @PutMapping("/{id}/updateEmployee")
    public Mono<ResponseEntity<Employee>> updateEmployee(@PathVariable Integer id, @RequestBody Mono<EmployeeDto> updatedEmployee) {
        return validator.isEmployeeIdValid(id)
                .flatMap(valid -> updatedEmployee
                        .flatMap(dto -> {
                            Employee employee = EmployeeMapper.toEntity(dto);
                            return validateEmployee(employee)
                                    .then(employeeService.updateEmployee(id, Mono.just(dto)));
                        })
                )
                .map(ResponseEntity::ok)
                .doOnError(error -> log.error("Error updating employee: {}", error.getMessage()));
    }

    @DeleteMapping("/deleteMapping/{id}")
    public Mono<ResponseEntity<Object>> deleteEmployee(@PathVariable Integer id){

        return validator.isEmployeeIdValid(id)
                .flatMap(valid -> employeeService.deleteEmployee(id))
                .thenReturn(ResponseEntity.ok().build())
                .doOnError(error -> log.error("Error deleting employee: {}", error.getMessage()));
    }

    private Mono<Void> validateEmployee(Employee employee) {
        return validator.isEmployeeNameValid(employee.getName())
                .then(validator.isEmployeeSalaryValid(employee.getSal()))
                .then();
    }
}
