package com.nisum.Employeeinfo.service;

import com.nisum.Employeeinfo.dto.EmployeeDto;
import com.nisum.Employeeinfo.mapper.EmployeeMapper;
import com.nisum.Employeeinfo.model.Employee;
import com.nisum.Employeeinfo.repository.EmployeeRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class EmployeeServiceImpl implements EmployeeService{

    @Autowired
    private EmployeeRepository repository;

    private static  final Logger log= LoggerFactory.getLogger(EmployeeServiceImpl.class);

    @Override
    public Mono<Employee> addEmployee(Mono<EmployeeDto> mono) {
        return mono
                .doOnNext(dto->log.info("received employee dto {}",dto))
                .map(EmployeeMapper::toEntity)
                .flatMap(employee -> repository.save(employee))
                .doOnNext(employee -> log.info("employee saved successfully {}",employee));
    }

    @Override
    public Flux<Employee> getAllEmployee() {
        return repository.findAll()
                .doOnNext(employee -> log.info("processing all the employees {}",employee))
                .doOnError(err->log.error("error occurred during fetching the employees  {}",err.getMessage(),err));
    }

    @Override
    public Mono<Employee> getEmployeeById(Integer id) {

        return repository.findById(id)
                .doOnSuccess(employee -> log.info("fin"));
    }

    @Override
    public Mono<Employee> updateEmployee(Integer id, Mono<EmployeeDto> updatedEmployeeMono) {

        return repository.findById(id)
                .flatMap(employee -> updatedEmployeeMono.flatMap(updatedEmployee->{
                    employee.setId(id);
                    employee.setName(updatedEmployee.getName());
                    employee.setDept(updatedEmployee.getDept());
                    employee.setEmail(updatedEmployee.getEmail());
                    employee.setSal(updatedEmployee.getSal());
                    return repository.save(employee);
                }))
                .switchIfEmpty(Mono.error(new RuntimeException("Employee Not Found")));

    }

    @Override
    public Mono<Void> deleteEmployee(Integer id) {

        return repository.deleteById(id)
                .doOnSuccess(success -> log.info("employee deleted successfully"))
                .doOnError(error -> log.error("Employee Not Found"));
    }
}
