package com.nisum.Employeeinfo.validator;

import com.nisum.Employeeinfo.errorcode.ApiErrorCode;
import com.nisum.Employeeinfo.exception.EmployeeNotFoundException;
import com.nisum.Employeeinfo.exception.InvalidEmployeeDataException;
import com.nisum.Employeeinfo.model.Employee;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

@Component
public class CommonServiceValidator {

    //validate if employee id is not null
    public Mono<Boolean> isEmployeeIdValid(Integer id) {
        if (id != null) {
            return Mono.just(true);
        }
        return Mono.error(new InvalidEmployeeDataException(ApiErrorCode.INVALID_EMPLOYEE_DATA, "Employee ID cannot be null"));
    }

    // Validate if Employee name is not null or empty
    public Mono<Boolean> isEmployeeNameValid(String name) {
        if (name != null && !name.trim().isEmpty()) {
            return Mono.just(true);
        }
        return Mono.error(new InvalidEmployeeDataException(ApiErrorCode.INVALID_EMPLOYEE_DATA, "Employee name cannot be empty"));
    }

    // Validate if Employee salary is non-negative
    public Mono<Boolean> isEmployeeSalaryValid(Double sal) {
        if (sal != null && sal >= 0) {
            return Mono.just(true);
        }
        return Mono.error(new InvalidEmployeeDataException(ApiErrorCode.INVALID_EMPLOYEE_DATA, "Employee salary must be non-negative"));
    }

    // Validate if the Employee object is valid as a whole
    public Mono<Boolean> isEmployeeObjectValid(Employee employee) {
        if (employee == null) {
            return Mono.error(new EmployeeNotFoundException(ApiErrorCode.EMPLOYEE_NOT_FOUND, "Employee object cannot be null"));
        }

        return isEmployeeIdValid(employee.getId())
                .flatMap(valid -> isEmployeeNameValid(employee.getName()))
                .flatMap(valid -> isEmployeeSalaryValid(employee.getSal()));
    }
}
